@@ -1,5 +1,76 @@
 ## function creates a stochastic character mapped tree as a modified "phylo" object
-## written by Liam Revell 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021
+## written by Liam Revell 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023
+
+## S3 method for Mk models & objects of various classes
+
+simmap<-function(object,...) UseMethod("simmap")
+
+simmap.default<-function(object,...){
+	warning(paste(
+		"simmap does not know how to handle objects of class ",
+		class(object),".\n"))
+}
+
+simmap.simmap<-function(object,...){
+	args<-list(...)
+	args$tree<-as.phylo(object)
+	args$x<-getStates(object,"tips")
+	if(hasArg(Q)) Q<-list(...)$Q
+	else {
+		args$Q<-if(!is.null(object$Q)) object$Q
+	}
+	args$pi<-if(hasArg(pi)) list(...)$pi else "fitzjohn"
+	if(hasArg(nsim)){
+		nsim<-list(...)$nsim
+		args$nsim<-nsim
+	} else args$nsim<-100
+	if(hasArg(trace)) trace<-list(...)$trace
+	else trace<-0
+	if(trace>0) args$message<-TRUE
+	else args$message<-FALSE
+	do.call(make.simmap,args)
+}
+
+simmap.fitpolyMk<-function(object,...) simmap.fitMk(object,...)
+
+simmap.fitMk<-function(object,...){
+	args<-list(...)
+	args$tree<-object$tree
+	args$x<-object$data
+	args$Q<-as.Qmatrix(object)
+	args$pi<-if(object$root.prior=="fitzjohn") 
+		"fitzjohn" else object$pi
+	if(is.null(args$nsim)) args$nsim<-100
+	if(hasArg(trace)) trace<-list(...)$trace
+	else trace<-0
+	if(trace>0) args$message<-TRUE
+	else args$message<-FALSE
+	do.call(make.simmap,args)
+}
+
+simmap.anova.fitMk<-function(object,...){
+	if(hasArg(weighted)) weighted<-list(...)$weighted
+	else weighted<-TRUE
+	if(hasArg(nsim)) nsim<-list(...)$nsim
+	else nsim<-100
+	if(weighted){
+		w<-object$weight
+		mods<-sample(1:length(w),size=nsim,replace=TRUE,prob=w)
+	} else {
+		best<-which(object$AIC==min(object$AIC))
+		mods<-sample(best,size=nsim,replace=TRUE)
+	}
+	fits<-attr(object,"models")[mods]
+	foo<-function(obj,args){
+		args$object<-obj
+		do.call(simmap,args)
+	}
+	args<-list(...)
+	args$nsim<-1
+	tt<-lapply(fits,foo,args=args)
+	class(tt)<-c("multiSimmap","multiPhylo")
+	return(tt)
+}
 
 make.simmap<-function(tree,x,model="SYM",nsim=1,...){
 	if(inherits(tree,"multiPhylo")){
@@ -8,8 +79,10 @@ make.simmap<-function(tree,x,model="SYM",nsim=1,...){
 			if(nsim>1) class(zz)<-NULL
 			return(zz)
 		}	
-		if(nsim>1) mtrees<-unlist(sapply(tree,ff,x,model,nsim,...,simplify=FALSE),recursive=FALSE)
-		else mtrees<-sapply(tree,ff,x,model,nsim,...,simplify=FALSE)
+		if(nsim>1){
+			mtrees<-unlist(sapply(tree,ff,x,model,nsim,...,simplify=FALSE),
+				recursive=FALSE)
+		} else mtrees<-sapply(tree,ff,x,model,nsim,...,simplify=FALSE)
 		class(mtrees)<-c("multiSimmap","multiPhylo")
 	} else {
 		## get optional arguments
@@ -42,7 +115,8 @@ make.simmap<-function(tree,x,model="SYM",nsim=1,...){
 		xx<-xx/rowSums(xx)
 		# reorder to cladewise
 		tree<-bt<-reorder.phylo(tree,"cladewise")
-		if(!is.binary(bt)) bt<-multi2di(bt,random=FALSE)
+		## this seems to be outdated & no longer needed
+		## if(!is.binary(bt)) bt<-multi2di(bt,random=FALSE)
 		# some preliminaries
 		N<-Ntip(tree)
 		m<-ncol(xx)
@@ -83,7 +157,8 @@ make.simmap<-function(tree,x,model="SYM",nsim=1,...){
 				list(tree=tree,x=x,N=N,m=m,root=root),SIMPLIFY=FALSE) else
 				list(smap(tree=tree,x=x,N=N,m=m,root=root,L=L[[1]],Q=Q[[1]],pi=pi[[1]],
 				logL=logL[[1]]))
-		} else if(is.matrix(Q)){
+		} else if(is(Q,"Qmatrix")||is.matrix(Q)){
+			if(is(Q,"Qmatrix")) Q<-unclass(Q)
 			XX<-getPars(bt,xx,model,Q=Q,tree,tol,m,pi=pi,args=list(...))
 			L<-XX$L
 			logL<-XX$loglik
@@ -197,7 +272,7 @@ mcmcQ<-function(bt,xx,model,tree,tol,m,burnin,samplefreq,nsim,vQ,prior,pi,args=l
 }
 
 # get pars
-# written by Liam J. Revell 2013, 2017, 2021
+# written by Liam J. Revell 2013, 2017, 2021, 2023
 getPars<-function(bt,xx,model,Q,tree,tol,m,liks=TRUE,pi,args=list()){
 	if(!is.null(args$pi)) args$pi<-NULL
 	args<-c(list(tree=bt,x=xx,model=model,fixedQ=Q,output.liks=liks,pi=pi),args)
@@ -209,11 +284,12 @@ getPars<-function(bt,xx,model,Q,tree,tol,m,liks=TRUE,pi,args=list()){
 	if(liks){
 		L<-obj$lik.anc
 		rownames(L)<-N+1:nrow(L)
-		if(!is.binary(tree)){
-			ancNames<-matchNodes(tree,bt)
-			L<-L[as.character(ancNames[,2]),]
-			rownames(L)<-ancNames[,1]
-		}
+		## this seems to be outdated & no longer needed
+		#if(!is.binary(tree)){
+		#	ancNames<-matchNodes(tree,bt)
+		#	L<-L[as.character(ancNames[,2]),]
+		#	rownames(L)<-ancNames[,1]
+		#}
 		L<-rbind(xx,L)
 		rownames(L)[1:N]<-1:N
 	} else L<-NULL	
@@ -245,8 +321,8 @@ smap<-function(tree,x,N,m,root,L,Q,pi,logL){
 		tree$edge[,2],sep=""),colnames(L)))
 	# now we want to simulate the node states & histories by pre-order traversal
 	NN<-matrix(NA,nrow(tree$edge),2) # our node values
-	NN[which(tree$edge[,1]==root),1]<-rstate(L[as.character(root),]*pi/
-		sum(L[as.character(root),]*pi)) # assign root
+	NN[which(tree$edge[,1]==root),1]<-rstate(L[as.character(root),]/
+		sum(L[as.character(root),])) # assign root
 	for(j in 1:nrow(tree$edge)){
 		# conditioned on the start value, assign end value of node (if internal)
 		p<-EXPM(Q*tree$edge.length[j])[NN[j,1],]*L[as.character(tree$edge[j,2]),]
@@ -372,20 +448,7 @@ density.multiSimmap<-function(x,...){
 		ab<-lapply(ab,function(x){
 			class(x)<-"mcmc"
 			x })
-		if(.check.pkg("coda")){
-			hpd.ab<-lapply(ab,HPDinterval)
-		} else {
-			cat("  HPDinterval requires package coda.\n")
-			cat("  Computing 95% interval from samples only.\n\n")
-			hpd95<-function(x){
-				obj<-setNames(c(sort(x)[round(0.025*length(x))],
-					sort(x)[round(0.975*length(x))]),
-					c("lower","upper"))
-				attr(obj,"Probability")<-0.95
-				obj
-			}
-			hpd.ab<-lapply(ab,hpd95)
-		}
+		hpd.ab<-lapply(ab,HPDinterval)
 		minmax<-range(unlist(ab))
 		pcalc<-function(x,mm)
 			hist(x,breaks=seq(mm[1]-1.5,mm[2]+1.5,bw),plot=FALSE)
@@ -413,13 +476,43 @@ density.multiSimmap<-function(x,...){
 plot.changesMap<-function(x,...){
 	if(hasArg(bty)) bty<-list(...)$bty
 	else bty<-"l"
+	if(hasArg(alpha)) alpha<-list(...)$alpha
+	else alpha<-0.3
+	if(hasArg(xlim)) xlim<-list(...)$xlim
+	else xlim<-NULL
+	if(hasArg(ylim)) ylim<-list(...)$ylim
+	else ylim<-NULL
+	if(hasArg(main)) main<-list(...)$main
+	else main<-NULL
+	if(hasArg(colors)){ 
+		colors<-list(...)$colors
+		nn<-names(colors)
+		colors<-setNames(make.transparent(colors,alpha),nn)
+	} else { 
+		colors<-if(length(x$trans)==2) 
+			setNames(make.transparent(c("blue","red"),alpha),x$trans)
+		else
+			setNames(rep(make.transparent("blue",alpha),length(x$trans)),
+				x$trans)
+	}
+	if(length(colors)<length(x$trans))
+		colors<-rep(colors,ceiling(length(x$trans)/length(colors)))[1:length(x$trans)]
+	if(is.null(names(colors))) colors<-setNames(colors,x$trans)
+	if(hasArg(transition)){ 
+		transition<-list(...)$transition
+		if(length(transition)>1){
+			cat("transition should be of length 1; truncating to first element.\n")
+			transition<-transition[1]
+		}
+	} else transition<-NULL
 	p<-x$p
 	hpd<-x$hpd
 	bw<-x$bw
-	if(length(x$trans)==2){
-		plot(p[[1]]$mids,p[[1]]$density,xlim=c(min(x$mins)-1,
-			max(x$maxs)+1),ylim=c(0,1.2*max(c(p[[1]]$density,
-			p[[2]]$density))),
+	if(length(x$trans)==2&&is.null(transition)){
+		plot(p[[1]]$mids,p[[1]]$density,xlim=if(is.null(xlim)) 
+			c(min(x$mins)-1,max(x$maxs)+1) else xlim,
+			ylim=if(is.null(ylim)) c(0,1.2*max(c(p[[1]]$density,
+			p[[2]]$density))) else ylim,
 			type="n",xlab="number of changes",
 			ylab="relative frequency across stochastic maps",
 			bty=bty)
@@ -428,56 +521,68 @@ plot.changesMap<-function(x,...){
 		x2<-rep(p[[1]]$mids-bw/2,each=2)[-1]
 		x3<-c(min(x2),x2,max(x2))
 		y3<-c(0,y2,0)
-		polygon(x3,y3,col=make.transparent("red",0.3),border=FALSE)
+		polygon(x3,y3,col=colors[x$trans[1]],border=FALSE)
 		lines(p[[1]]$mids-bw/2,p[[1]]$density,type="s")
 		y2<-rep(p[[2]]$density,each=2)
 		y2<-y2[-length(y2)]
 		x2<-rep(p[[2]]$mids-bw/2,each=2)[-1]
 		x3<-c(min(x2),x2,max(x2))
 		y3<-c(0,y2,0)
-		polygon(x3,y3,col=make.transparent("blue",0.3),border=FALSE)
+		polygon(x3,y3,col=colors[x$trans[2]],border=FALSE)
 		lines(p[[2]]$mids-bw/2,p[[2]]$density,type="s")
-		add.simmap.legend(colors=setNames(c(make.transparent("red",0.3),
-			make.transparent("blue",0.3)),x$trans[1:2]),
-			prompt=FALSE,x=min(x$mins),y=0.95*par()$usr[4])
 		dd<-0.01*diff(par()$usr[3:4])
 		lines(hpd[[1]],rep(max(p[[1]]$density)+dd,2))
 		lines(rep(hpd[[1]][1],2),c(max(p[[1]]$density)+dd,
 			max(p[[1]]$density)+dd-0.005))
 		lines(rep(hpd[[1]][2],2),c(max(p[[1]]$density)+dd,
 			max(p[[1]]$density)+dd-0.005))
+		CHARS<-strsplit(x$trans[1],"->")[[1]]
+		CHARS[1]<-paste("HPD(",CHARS[1],collapse="")
+		CHARS[2]<-paste(CHARS[2],")",collapse="")
+		T1<-bquote(.(CHARS[1])%->%.(CHARS[2]))
 		text(mean(hpd[[1]]),max(p[[1]]$density)+dd,
-			paste("HPD(",x$trans[1],")",sep=""),
-			pos=3)
+			T1,pos=3)
 		lines(hpd[[2]],rep(max(p[[2]]$density)+dd,2))
 		lines(rep(hpd[[2]][1],2),c(max(p[[2]]$density)+dd,
 			max(p[[2]]$density)+dd-0.005))
 		lines(rep(hpd[[2]][2],2),c(max(p[[2]]$density)+dd,
 			max(p[[2]]$density)+dd-0.005))
+		CHARS<-strsplit(x$trans[2],"->")[[1]]
+		CHARS[1]<-paste("HPD(",CHARS[1],collapse="")
+		CHARS[2]<-paste(CHARS[2],")",collapse="")
+		T2<-bquote(.(CHARS[1])%->%.(CHARS[2]))
 		text(mean(hpd[[2]]),max(p[[2]]$density)+dd,
-			paste("HPD(",x$trans[2],")",sep=""),
-			pos=3)
+			T2,pos=3)
+		CHARS<-strsplit(x$trans[1],"->")[[1]]
+		T1<-bquote(.(CHARS[1])%->%.(CHARS[2]))
+		CHARS<-strsplit(x$trans[2],"->")[[1]]
+		T2<-bquote(.(CHARS[1])%->%.(CHARS[2]))
+		legend("topleft",legend=c(T1,T2),pch=22,pt.cex=2.2,bty="n",
+			pt.bg=colors[x$trans])
 	} else {
-		k<-length(x$states)
-		par(mfrow=c(k,k))
-		ii<-1
+		k<-if(is.null(transition)) length(x$states) else 1
+		if(k>1) par(mfrow=c(k,k))
+		ii<-if(is.null(transition)) 1 else which(x$trans==transition)
 		max.d<-max(unlist(lapply(p,function(x) x$density)))
 		for(i in 1:k){
 			for(j in 1:k){
-				if(i==j) plot.new()
+				if(i==j&&is.null(transition)) plot.new()
 				else {
-					plot(p[[ii]]$mids,p[[ii]]$density,xlim=c(min(x$mins)-1,
-						max(x$maxs)+1),ylim=c(0,1.2*max.d),
+					CHARS<-strsplit(x$trans[ii],"->")[[1]]
+					MAIN<-if(is.null(main)) bquote(.(CHARS[1])%->%.(CHARS[2])) else
+						main
+					plot(p[[ii]]$mids,p[[ii]]$density,xlim=if(is.null(xlim)) 
+						c(min(x$mins)-1,max(x$maxs)+1) else xlim,
+						ylim=if(is.null(ylim)) c(0,1.2*max.d) else ylim,
 						type="n",xlab="number of changes",
-						ylab="relative frequency",main=x$trans[ii],font.main=1,
+						ylab="relative frequency",main=MAIN,font.main=1,
 						bty=bty)
-					##title(main=)
 					y2<-rep(p[[ii]]$density,each=2)
 					y2<-y2[-length(y2)]
 					x2<-rep(p[[ii]]$mids-bw/2,each=2)[-1]
 					x3<-c(min(x2),x2,max(x2))
 					y3<-c(0,y2,0)
-					polygon(x3,y3,col=make.transparent("blue",0.3),border=FALSE)
+					polygon(x3,y3,col=colors[x$trans[ii]],border=FALSE)
 					lines(p[[ii]]$mids-bw/2,p[[ii]]$density,type="s")
 					dd<-0.03*diff(par()$usr[3:4])
 					lines(hpd[[ii]],rep(max(p[[ii]]$density)+dd,2))
